@@ -77,6 +77,9 @@ irc_nickserv (server *serv, char *pass)
 		break;
 	case 2:
 		tcp_sendf (serv, "NS :identify %s\r\n", pass);
+		break;
+	case 3:
+		tcp_sendf (serv, "PRIVMSG NS :identify %s\r\n", pass);
 	}
 }
 
@@ -611,6 +614,20 @@ process_numeric (session * sess, int n,
 		}
 		break;
 
+	case 348:	/* +e-list entry */
+		if (!inbound_banlist (sess, atol (word[7]), word[4], word[5], word[6], TRUE))
+			goto def;
+		break;
+
+	case 349:	/* end of exemption list */
+		sess = find_channel (serv, word[4]);
+		if (!sess)
+			sess = serv->front_session;
+		if (!fe_is_banwindow (sess))
+			goto def;
+		fe_ban_list_end (sess, TRUE);
+		break;
+
 	case 353:						  /* NAMES */
 		inbound_nameslist (serv, word[5],
 							(word_eol[6][0] == ':') ? word_eol[6] + 1 : word_eol[6]);
@@ -622,7 +639,7 @@ process_numeric (session * sess, int n,
 		break;
 
 	case 367: /* banlist entry */
-		inbound_banlist (sess, atol (word[7]), word[4], word[5], word[6]);
+		inbound_banlist (sess, atol (word[7]), word[4], word[5], word[6], FALSE);
 		break;
 
 	case 368:
@@ -631,7 +648,7 @@ process_numeric (session * sess, int n,
 			sess = serv->front_session;
 		if (!fe_is_banwindow (sess))
 			goto def;
-		fe_ban_list_end (sess);
+		fe_ban_list_end (sess, FALSE);
 		break;
 
 	case 372:	/* motd text */
@@ -688,6 +705,16 @@ process_numeric (session * sess, int n,
 		break;
 
 	default:
+
+		if (serv->inside_whois && word[4][0])
+		{
+			/* some unknown WHOIS reply, ircd coders make them up weekly */
+			EMIT_SIGNAL (XP_TE_WHOIS_SPECIAL, serv->server_session, word[4],
+							(word_eol[5][0] == ':') ? word_eol[5] + 1 : word_eol[5],
+							 word[2], NULL, 0);
+			return;
+		}
+
 	def:
 		if (is_channel (serv, word[4]))
 		{
