@@ -15,6 +15,9 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
+
+#define __APPLE_API_STRICT_CONFORMANCE
+
 #define _FILE_OFFSET_BITS 64
 #include <stdio.h>
 #include <unistd.h>
@@ -40,9 +43,10 @@
 #include "util.h"
 #include "../../config.h"
 
+#define WANTSOCKET
 #include "inet.h"
 
-#ifdef USING_FREEBSD
+#if defined (USING_FREEBSD) || defined (__APPLE__)
 #include <sys/sysctl.h>
 #endif
 #ifdef SOCKS
@@ -111,7 +115,7 @@ xchat_realloc (char *old, int len, char *file, int line)
 	if (ret)
 	{
 		strcpy (ret, old);
-		xchat_free (old, file, line);
+		xchat_dfree (old, file, line);
 	}
 	return ret;
 }
@@ -189,7 +193,7 @@ xchat_mem_list (void)
 }
 
 void
-xchat_free (void *buf, char *file, int line)
+xchat_dfree (void *buf, char *file, int line)
 {
 	struct mem_block *cur, *last;
 
@@ -229,7 +233,7 @@ xchat_free (void *buf, char *file, int line)
 
 #define malloc(n) xchat_malloc(n, __FILE__, __LINE__)
 #define realloc(n, m) xchat_realloc(n, m, __FILE__, __LINE__)
-#define free(n) xchat_free(n, __FILE__, __LINE__)
+#define free(n) xchat_dfree(n, __FILE__, __LINE__)
 #define strdup(n) xchat_strdup(n, __FILE__, __LINE__)
 
 #endif /* MEMORY_DEBUG */
@@ -354,14 +358,21 @@ errorstring (int err)
 }
 
 int
-waitline (int sok, char *buf, int bufsize)
+waitline (int sok, char *buf, int bufsize, int use_recv)
 {
 	int i = 0;
 
 	while (1)
 	{
-		if (read (sok, &buf[i], 1) < 1)
-			return -1;
+		if (use_recv)
+		{
+			if (recv (sok, &buf[i], 1, 0) < 1)
+				return -1;
+		} else
+		{
+			if (read (sok, &buf[i], 1) < 1)
+				return -1;
+		}
 		if (buf[i] == '\n' || bufsize == i + 1)
 		{
 			buf[i] = 0;
@@ -410,13 +421,17 @@ expand_homedir (char *file)
 }
 
 char *
-strip_color (char *text)
+strip_color (char *text, int len, int do_color, int do_attr)
 {
 	int nc = 0;
 	int i = 0;
 	int col = 0;
-	int len = strlen (text);
-	char *new_str = malloc (len + 2);
+	char *new_str;
+
+	if (len == -1)
+		len = strlen (text);
+
+	new_str = malloc (len + 2);
 
 	while (len > 0)
 	{
@@ -426,6 +441,11 @@ strip_color (char *text)
 			nc++;
 			if (*text == ',')
 				nc = 0;
+			if (!do_color)
+			{
+				new_str[i] = *text;
+				i++;
+			}
 		} else
 		{
 			col = 0;
@@ -434,12 +454,22 @@ strip_color (char *text)
 			case '\003':			  /*ATTR_COLOR: */
 				col = 1;
 				nc = 0;
+				if (!do_color)
+				{
+					new_str[i] = *text;
+					i++;
+				}
 				break;
 			case '\007':			  /*ATTR_BEEP: */
 			case '\017':			  /*ATTR_RESET: */
 			case '\026':			  /*ATTR_REVERSE: */
 			case '\002':			  /*ATTR_BOLD: */
 			case '\037':			  /*ATTR_UNDERLINE: */
+				if (!do_attr)
+				{
+					new_str[i] = *text;
+					i++;
+				}
 				break;
 			default:
 				new_str[i] = *text;
@@ -455,7 +485,7 @@ strip_color (char *text)
 	return new_str;
 }
 
-#if defined (USING_LINUX) || defined (USING_FREEBSD)
+#if defined (USING_LINUX) || defined (USING_FREEBSD) || defined (__APPLE__)
 
 static void
 get_cpu_info (double *mhz, int *cpus)
@@ -478,7 +508,7 @@ get_cpu_info (double *mhz, int *cpus)
 
 	while (1)
 	{
-		if (waitline (fh, buf, sizeof buf) < 0)
+		if (waitline (fh, buf, sizeof buf, FALSE) < 0)
 			break;
 		if (!strncmp (buf, "cycle frequency [Hz]\t:", 22))	/* alpha */
 		{
@@ -505,6 +535,7 @@ get_cpu_info (double *mhz, int *cpus)
 	u_long freq;
 	size_t len;
 
+	freq = 0;
 	*mhz = 0;
 	*cpus = 0;
 
@@ -516,7 +547,30 @@ get_cpu_info (double *mhz, int *cpus)
 
 	len = sizeof(freq);
 	sysctlbyname("machdep.tsc_freq", &freq, &len, NULL, 0);
-	
+
+	*cpus = ncpu;
+	*mhz = (freq / 1000000);
+
+#endif
+#ifdef __APPLE__
+
+	int mib[2], ncpu;
+	unsigned long long freq;
+	size_t len;
+
+	freq = 0;
+	*mhz = 0;
+	*cpus = 0;
+
+	mib[0] = CTL_HW;
+	mib[1] = HW_NCPU;
+
+	len = sizeof(ncpu);
+	sysctl(mib, 2, &ncpu, &len, NULL, 0);
+
+	len = sizeof(freq);
+        sysctlbyname("hw.cpufrequency", &freq, &len, NULL, 0);
+
 	*cpus = ncpu;
 	*mhz = (freq / 1000000);
 
@@ -577,10 +631,10 @@ get_cpu_str (void)
 char *
 get_cpu_str (void)
 {
-#if defined (USING_LINUX) || defined (USING_FREEBSD)
+#if defined (USING_LINUX) || defined (USING_FREEBSD) || defined (__APPLE__)
 	double mhz;
-	int cpus;
 #endif
+	int cpus = 1;
 	struct utsname un;
 	static char *buf = NULL;
 
@@ -591,7 +645,7 @@ get_cpu_str (void)
 
 	uname (&un);
 
-#if defined (USING_LINUX) || defined (USING_FREEBSD)
+#if defined (USING_LINUX) || defined (USING_FREEBSD) || defined (__APPLE__)
 	get_cpu_info (&mhz, &cpus);
 	if (mhz)
 	{
@@ -603,7 +657,9 @@ get_cpu_str (void)
 					cpuspeed, cpuspeedstr);
 	} else
 #endif
-		snprintf (buf, 128, "%s %s [%s]", un.sysname, un.release, un.machine);
+		snprintf (buf, 128,
+					(cpus == 1) ? "%s %s [%s]" : "%s %s [%s/SMP]",
+					un.sysname, un.release, un.machine);
 
 	return buf;
 }
@@ -749,25 +805,19 @@ tolowerStr (char *str)
 	}
 }*/
 
-/* This is originally from BitchX, but since added missing TLDs and
-   changed to a binary search */
-
 typedef struct
 {
-	char *code;
-	char *country;
-} Domain;
+	char *code, *country;
+} domain_t;
 
 static int
 country_compare (const void *a, const void *b)
 {
-	return strcasecmp (a, ((Domain *)b)->code);
+	return strcasecmp (a, ((domain_t *)b)->code);
 }
 
-char *
-country (char *hostname)
+static const domain_t domain[] =
 {
-	static const Domain domain[] = {
 		{"AD", N_("Andorra") },
 		{"AE", N_("United Arab Emirates") },
 		{"AF", N_("Afghanistan") },
@@ -808,10 +858,11 @@ country (char *hostname)
 		{"BZ", N_("Belize") },
 		{"CA", N_("Canada") },
 		{"CC", N_("Cocos Islands") },
+		{"CD", N_("Democratic Republic of Congo") },
 		{"CF", N_("Central African Republic") },
 		{"CG", N_("Congo") },
 		{"CH", N_("Switzerland") },
-		{"CI", N_("Cote D'ivoire") },
+		{"CI", N_("Cote d'Ivoire") },
 		{"CK", N_("Cook Islands") },
 		{"CL", N_("Chile") },
 		{"CM", N_("Cameroon") },
@@ -819,7 +870,7 @@ country (char *hostname)
 		{"CO", N_("Colombia") },
 		{"COM", N_("Internic Commercial") },
 		{"CR", N_("Costa Rica") },
-		{"CS", N_("Former Czechoslovakia") },
+		{"CS", N_("Serbia and Montenegro") },
 		{"CU", N_("Cuba") },
 		{"CV", N_("Cape Verde") },
 		{"CX", N_("Christmas Island") },
@@ -861,7 +912,7 @@ country (char *hostname)
 		{"GP", N_("Guadeloupe") },
 		{"GQ", N_("Equatorial Guinea") },
 		{"GR", N_("Greece") },
-		{"GS", N_("S. Georgia and S. Sandwich Isles.") },
+		{"GS", N_("S. Georgia and S. Sandwich Isles") },
 		{"GT", N_("Guatemala") },
 		{"GU", N_("Guam") },
 		{"GW", N_("Guinea-Bissau") },
@@ -957,6 +1008,7 @@ country (char *hostname)
 		{"PM", N_("St. Pierre and Miquelon") },
 		{"PN", N_("Pitcairn") },
 		{"PR", N_("Puerto Rico") },
+		{"PS", N_("Palestinian Territory") },
 		{"PT", N_("Portugal") },
 		{"PW", N_("Palau") },
 		{"PY", N_("Paraguay") },
@@ -1010,7 +1062,7 @@ country (char *hostname)
 		{"UY", N_("Uruguay") },
 		{"UZ", N_("Uzbekistan") },
 		{"VA", N_("Vatican City State") },
-		{"VC", N_("St. Vincent and the grenadines") },
+		{"VC", N_("St. Vincent and the Grenadines") },
 		{"VE", N_("Venezuela") },
 		{"VG", N_("British Virgin Islands") },
 		{"VI", N_("US Virgin Islands") },
@@ -1023,11 +1075,14 @@ country (char *hostname)
 		{"YU", N_("Yugoslavia") },
 		{"ZA", N_("South Africa") },
 		{"ZM", N_("Zambia") },
-		{"ZR", N_("Zaire") },
 		{"ZW", N_("Zimbabwe") },
-	};
+};
+
+char *
+country (char *hostname)
+{
 	char *p;
-	Domain *dom;
+	domain_t *dom;
 
 	if (!hostname || !*hostname || isdigit (hostname[strlen (hostname) - 1]))
 		return _("Unknown");
@@ -1036,8 +1091,8 @@ country (char *hostname)
 	else
 		p = hostname;
 
-	dom = bsearch (p, domain, sizeof (domain) / sizeof (Domain),
-						sizeof (Domain), country_compare);
+	dom = bsearch (p, domain, sizeof (domain) / sizeof (domain_t),
+						sizeof (domain_t), country_compare);
 
 	if (!dom)
 		return _("Unknown");
@@ -1045,6 +1100,22 @@ country (char *hostname)
 	return _(dom->country);
 }
 
+void
+country_search (char *pattern, void *ud, void (*print)(void *, char *, ...))
+{
+	const domain_t *dom;
+	int i;
+
+	for (i = 0; i < sizeof (domain) / sizeof (domain_t); i++)
+	{
+		dom = &domain[i];
+		if (match (pattern, dom->country) || match (pattern, _(dom->country)))
+		{
+			print (ud, "%s = %s\n", dom->code, _(dom->country));
+		}
+	}
+}
+
 /* I think gnome1.0.x isn't necessarily linked against popt, ah well! */
 /* !!! For now use this inlined function, or it would break fe-text building */
 /* .... will find a better solution later. */
@@ -1145,6 +1216,7 @@ util_exec (char *cmd)
 	int pid;
 	char **argv;
 	int argc;
+	int fd;
 
 	if (my_poptParseArgvString (cmd, &argc, &argv) != 0)
 		return -1;
@@ -1155,6 +1227,8 @@ util_exec (char *cmd)
 		return -1;
 	if (pid == 0)
 	{
+		/* Now close all open file descriptors except stdin, stdout and stderr */
+		for (fd = 3; fd < 1024; fd++) close(fd);
 		execvp (argv[0], argv);
 		_exit (0);
 	} else
@@ -1466,3 +1540,22 @@ download_move_to_completed_dir (char *dcc_dir, char *dcc_completed_dir,
 			unlink (dl_src);
 	}
 }
+
+int
+mkdir_utf8 (char *dir)
+{
+	int ret;
+
+	dir = g_filename_from_utf8 (dir, -1, 0, 0, 0);
+	if (!dir)
+		return -1;
+
+#ifdef WIN32
+	ret = mkdir (dir);
+#else
+	ret = mkdir (dir, S_IRUSR | S_IWUSR | S_IXUSR);
+#endif
+	g_free (dir);
+
+	return ret;
+}
