@@ -23,11 +23,11 @@
 #ifdef USE_DEBUG
 #define malloc(n) xchat_malloc(n, __FILE__, __LINE__)
 #define realloc(n, m) xchat_realloc(n, m, __FILE__, __LINE__)
-#define free(n) xchat_free(n, __FILE__, __LINE__)
+#define free(n) xchat_dfree(n, __FILE__, __LINE__)
 #define strdup(n) xchat_strdup(n, __FILE__, __LINE__)
 void *xchat_malloc (int size, char *file, int line);
 void *xchat_strdup (char *str, char *file, int line);
-void xchat_free (void *buf, char *file, int line);
+void xchat_dfree (void *buf, char *file, int line);
 void *xchat_realloc (char *old, int len, char *file, int line);
 #endif
 
@@ -73,9 +73,10 @@ void *xchat_realloc (char *old, int len, char *file, int line);
 #define NICKLEN		64				/* including the NULL, so 63 really */
 #define CHANLEN		300
 #define PDIWORDS		32
+#define USERNAMELEN 10
 
-#define safe_strcpy(dest,src,len)	{strncpy(dest,src,len); \
-												dest[len-1] = 0;}
+#define safe_strcpy(dest,src,len)	{strncpy((dest),(src),(len)); \
+												(dest)[len-1] = 0;}
 
 #if defined(ENABLE_NLS) && !defined(_)
 #  include <libintl.h>
@@ -123,7 +124,9 @@ struct xchatprefs
 	char background[PATHLEN + 1];
 	char dccdir[PATHLEN + 1];
 	char dcc_completed_dir[PATHLEN + 1];
-	char bluestring[300];
+	char irc_extra_hilight[300];
+	char irc_no_hilight[300];
+	char irc_nick_hilight[300];
 	char dnsprogram[72];
 	char hostname[127];
 	char cmdchar[4];
@@ -293,8 +296,8 @@ typedef struct session
 	void *usertree;					/* ordered with Ops first */
 	struct User *me;					/* points to myself in the usertree */
 	char channel[CHANLEN];
-	char waitchannel[CHANLEN];		  /* waiting to join this channel */
-	char willjoinchannel[CHANLEN];	  /* /join done for this channel */
+	char waitchannel[CHANLEN];		  /* waiting to join channel (/join sent) */
+	char willjoinchannel[CHANLEN];	  /* will issue /join for this channel */
 	char channelkey[64];			  /* XXX correct max length? */
 	int limit;						  /* channel user limit */
 	int logfd;
@@ -452,6 +455,8 @@ typedef struct server
 	unsigned int have_whox:1;	/* have undernet's WHOX features */
 	unsigned int have_capab:1;	/* supports CAPAB (005 tells us) */
 	unsigned int have_idmsg:1;	/* freenode's IDENTIFY-MSG */
+	unsigned int have_except:1;	/* ban exemptions +e */
+	unsigned int using_cp1255:1;	/* encoding is CP1255/WINDOWS-1255? */
 	int use_who:1;				/* whether to use WHO command to get dcc_ip */
 #ifdef USE_OPENSSL
 	int use_ssl:1;					  /* is server SSL capable? */
@@ -468,6 +473,7 @@ struct commands
 	cmd_callback callback;
 	char needserver;
 	char needchannel;
+	char handle_quotes;
 	char *help;
 };
 
