@@ -47,6 +47,7 @@
 #include "../common/outbound.h"
 #include "../common/util.h"
 #include "../common/text.h"
+#include "../common/plugin.h"
 #include <gdk/gdkkeysyms.h>
 #include "gtkutil.h"
 #include "menu.h"
@@ -57,13 +58,8 @@
 #include "fkeys.h"
 
 
-static void tab_comp_chan (session *sess, GtkWidget * t);
-static void tab_comp_cmd (session *sess, GtkWidget * t);
-static int tab_nick_comp (session *sess, GtkWidget * t, int shift);
-static void nick_comp_chng (session *sess, GtkWidget * t, int updown);
 static void replace_handle (GtkWidget * wid);
-static gint alpha_string_compare (gconstpointer, gconstpointer);
-
+void key_action_tab_clean (void);
 
 /***************** Key Binding Code ******************/
 
@@ -104,6 +100,12 @@ struct key_action
 	char *help;
 };
 
+struct gcomp_data
+{
+	char data[CHANLEN];
+	int elen;
+};
+
 static int key_load_kbs (char *);
 static void key_load_defaults ();
 static void key_save_kbs (char *);
@@ -125,7 +127,7 @@ static int key_action_history_down (GtkWidget * wid, GdkEventKey * evt,
 static int key_action_tab_comp (GtkWidget * wid, GdkEventKey * evt, char *d1,
 										  char *d2, struct session *sess);
 static int key_action_comp_chng (GtkWidget * wid, GdkEventKey * evt, char *d1,
-											char *d2, struct session *sess);
+                                                                                        char *d2, struct session *sess);
 static int key_action_replace (GtkWidget * wid, GdkEventKey * evt, char *d1,
 										 char *d2, struct session *sess);
 static int key_action_move_tab_left (GtkWidget * wid, GdkEventKey * evt,
@@ -156,7 +158,7 @@ static const struct key_action key_actions[KEY_MAX_ACTIONS + 1] = {
 	{key_action_insert, "Insert in Buffer",
 	 N_("The \002Insert in Buffer\002 command will insert the contents of Data 1 into the entry where the key sequence was pressed at the current cursor position")},
 	{key_action_scroll_page, "Scroll Page",
-	 N_("The \002Scroll Page\002 command scrolls the text widget up or down one page. If Data 1 is set to anything the page scrolls up, else it scrolls down")},
+	 N_("The \002Scroll Page\002 command scrolls the text widget up or down one page or one line. Set Data 1 to either Up, Down, +1 or -1.")},
 	{key_action_set_buffer, "Set Buffer",
 	 N_("The \002Set Buffer\002 command sets the entry where the key sequence was entered to the contents of Data 1")},
 	{key_action_history_up, "Last Command",
@@ -262,6 +264,13 @@ key_handle_key_press (GtkWidget *wid, GdkEventKey *evt, session *sess)
 		return FALSE;
 	current_sess = sess;
 
+	if (plugin_emit_keypress (sess, evt->state, evt->keyval, evt->length, evt->string))
+		return 1;
+
+	/* maybe the plugin closed this tab? */
+	if (!is_session (sess))
+		return 1;
+
 	mod = evt->state & (STATE_CTRL | STATE_ALT | STATE_SHIFT);
 
 	kb = keys_root;
@@ -295,6 +304,8 @@ key_handle_key_press (GtkWidget *wid, GdkEventKey *evt, session *sess)
 		last = kb;
 		kb = kb->next;
 	}
+	if (keyval == GDK_space)
+		key_action_tab_clean ();
 
 	/* check if it's a return or enter */
 	/* ---handled by the "activate" signal in maingui.c */
@@ -379,13 +390,15 @@ key_load_defaults ()
 		"A\n3\nChange Page\nD1:3\nD2!\n\n"\
 		"A\n2\nChange Page\nD1:2\nD2!\n\n"\
 		"A\n1\nChange Page\nD1:1\nD2!\n\n"\
-		"C\no\nInsert in Buffer\nD1:%O\nD2!\n\n"\
-		"C\nb\nInsert in Buffer\nD1:%B\nD2!\n\n"\
-		"C\nk\nInsert in Buffer\nD1:%C\nD2!\n\n"\
+		"C\no\nInsert in Buffer\nD1:\nD2!\n\n"\
+		"C\nb\nInsert in Buffer\nD1:\nD2!\n\n"\
+		"C\nk\nInsert in Buffer\nD1:\nD2!\n\n"\
 		"S\nNext\nChange Selected Nick\nD1!\nD2!\n\n"\
 		"S\nPrior\nChange Selected Nick\nD1:Up\nD2!\n\n"\
-		"None\nNext\nScroll Page\nD1!\nD2!\n\n"\
+		"None\nNext\nScroll Page\nD1:Down\nD2!\n\n"\
 		"None\nPrior\nScroll Page\nD1:Up\nD2!\n\n"\
+		"S\nDown\nScroll Page\nD1:+1\nD2!\n\n"\
+		"S\nUp\nScroll Page\nD1:-1\nD2!\n\n"\
 		"None\nDown\nNext Command\nD1!\nD2!\n\n"\
 		"None\nUp\nLast Command\nD1!\nD2!\n\n"\
 		"None\nTab\nComplete nick/command\nD1!\nD2!\n\n"\
@@ -396,7 +409,8 @@ key_load_defaults ()
 		"A\nLeft\nMove front tab left\nD1!\nD2!\n\n"\
 		"A\nRight\nMove front tab right\nD1!\nD2!\n\n"\
 		"CS\nPrior\nMove tab family left\nD1!\nD2!\n\n"\
-		"CS\nNext\nMove tab family right\nD1!\nD2!\n\n"
+		"CS\nNext\nMove tab family right\nD1!\nD2!\n\n"\
+		"None\nF9\nRun Command\nD1:/GUI MENU TOGGLE\nD2!\n\n"
 	char buf[512];
 	int fd;
 
@@ -683,7 +697,7 @@ key_dialog_show ()
 	}
 
 	key_dialog =
-			  mg_create_generic_tab ("editkeys", _("X-Chat: Edit Key Bindings"),
+			  mg_create_generic_tab ("editkeys", _("X-Chat: Keyboard Shortcuts"),
 							TRUE, FALSE, key_dialog_close, NULL, 560, 330, &vbox, 0);
 
 	hbox = gtk_hbox_new (0, 2);
@@ -1225,28 +1239,51 @@ key_action_scroll_page (GtkWidget * wid, GdkEventKey * evt, char *d1,
 {
 	int value, end;
 	GtkAdjustment *adj;
-	int up = 0;
+	enum scroll_type { PAGE_UP, PAGE_DOWN, LINE_UP, LINE_DOWN };
+	int type = PAGE_DOWN;
 
-	if (d1 && d1[0] != 0)
-		up++;
-
-	if (sess)
+	if (d1)
 	{
-		adj = GTK_RANGE (sess->gui->vscrollbar)->adjustment;
-		if (up)						  /* PageUp */
-		{
-			value = adj->value - (adj->page_size - 1);
-			if (value < 0)
-				value = 0;
-		} else
-		{								  /* PageDown */
-			end = adj->upper - adj->lower - adj->page_size;
-			value = adj->value + (adj->page_size - 1);
-			if (value > end)
-				value = end;
-		}
-		gtk_adjustment_set_value (adj, value);
+		if (!strcasecmp (d1, "up"))
+			type = PAGE_UP;
+		else if (!strcasecmp (d1, "+1"))
+			type = LINE_DOWN;
+		else if (!strcasecmp (d1, "-1"))
+			type = LINE_UP;
 	}
+
+	if (!sess)
+		return 0;
+
+	adj = GTK_RANGE (sess->gui->vscrollbar)->adjustment;
+	end = adj->upper - adj->lower - adj->page_size;
+
+	switch (type)
+	{
+	case LINE_UP:
+		value = adj->value - 1.0;
+		break;
+
+	case LINE_DOWN:
+		value = adj->value + 1.0;
+		break;
+
+	case PAGE_UP:
+		value = adj->value - (adj->page_size - 1);
+		break;
+
+	default:	/* PAGE_DOWN */
+		value = adj->value + (adj->page_size - 1);
+		break;
+	}
+
+	if (value < 0)
+		value = 0;
+	if (value > end)
+		value = end;
+
+	gtk_adjustment_set_value (adj, value);
+
 	return 0;
 }
 
@@ -1297,75 +1334,327 @@ key_action_history_down (GtkWidget * wid, GdkEventKey * ent, char *d1,
 	return 2;
 }
 
+/* old data that we reuse */
+static struct gcomp_data old_gcomp;
+
+/* work on the data, ie return only channels */
 static int
-key_action_tab_comp (GtkWidget * wid, GdkEventKey * ent, char *d1, char *d2,
+double_chan_cb (session *lsess, GList **list)
+{
+	if (lsess->type == SESS_CHANNEL)
+		*list = g_list_prepend(*list, lsess->channel);
+	return TRUE;
+}
+
+/* convert a slist -> list. */
+static GList *
+chanlist_double_list (GSList *inlist)
+{
+	GList *list = NULL;
+	g_slist_foreach(inlist, (GFunc)double_chan_cb, &list);
+	return list;
+}
+
+/* handle commands */
+static int
+double_cmd_cb (struct popup *pop, GList **list)
+{
+	*list = g_list_prepend(*list, pop->name);
+	return TRUE;
+}
+
+/* convert a slist -> list. */
+static GList *
+cmdlist_double_list (GSList *inlist)
+{
+	GList *list = NULL;
+	g_slist_foreach (inlist, (GFunc)double_cmd_cb, &list);
+	return list;
+}
+
+static char *
+gcomp_nick_func (char *data)
+{
+	if (data)
+		return ((struct User *)data)->nick;
+	return "";
+}
+
+void
+key_action_tab_clean(void)
+{
+	if (old_gcomp.elen)
+	{
+		old_gcomp.data[0] = 0;
+		old_gcomp.elen = 0;
+	}
+}
+
+/* Used in the followig completers */
+#define COMP_BUF 2048
+
+static int
+key_action_tab_comp (GtkWidget *t, GdkEventKey *entry, char *d1, char *d2,
 							struct session *sess)
 {
+	int len = 0, elen = 0, i = 0, cursor_pos, ent_start = 0, comp = 0, found = 0,
+	    prefix_len, skip_len = 0, is_nick, is_cmd = 0;
+	char buf[COMP_BUF], ent[CHANLEN], *postfix = NULL, *result, *ch;
+	GList *list = NULL, *tmp_list = NULL;
 	const char *text;
-	int chan_start, cursor_pos;
+	GCompletion *gcomp = NULL;
 
-	/* obtain the entered text and cursor position */
-	text = gtk_entry_get_text (GTK_ENTRY (wid));
-	cursor_pos = gtk_editable_get_position (GTK_EDITABLE (wid));
+	/* force the IM Context to reset */
+	gtk_editable_set_editable (GTK_EDITABLE (t), FALSE);
+	gtk_editable_set_editable (GTK_EDITABLE (t), TRUE);
 
-	/* if no text has been entered, do nothing */
-	if (cursor_pos == 0)
+	text = GTK_ENTRY (t)->text;
+	if (text[0] == 0)
 		return 1;
 
-	/* search backwards to find the /, #, space or start */
-	for (chan_start = cursor_pos; chan_start >= 0; --chan_start)
+	len = g_utf8_strlen (text, -1); /* must be null terminated */
+
+	cursor_pos = gtk_editable_get_position (GTK_EDITABLE (t));
+
+	buf[0] = 0; /* make sure we don't get garbage in the buffer */
+
+	/* handle "nick: " or "nick " or "#channel "*/
+	ch = g_utf8_find_prev_char(text, g_utf8_offset_to_pointer(text,cursor_pos));
+	if (ch && ch[0] == ' ')
 	{
-		/* check if we can match a channel */
-		if (text[chan_start] == '#')
+		skip_len++;
+		ch = g_utf8_find_prev_char(text, ch);
+		if (!ch)
+			return 2;
+
+		cursor_pos = g_utf8_pointer_to_offset(text, ch);
+		if (cursor_pos && (g_utf8_get_char_validated(ch, -1) == ':' || 
+					g_utf8_get_char_validated(ch, -1) == ',' ||
+					g_utf8_get_char_validated(ch, -1) == prefs.nick_suffix[0]))
 		{
-			if (chan_start == 0 || text[chan_start - 1] == ' ')
+			skip_len++;
+		}
+		else
+			cursor_pos = g_utf8_pointer_to_offset(text, g_utf8_offset_to_pointer(ch, 1));
+	}
+
+	comp = skip_len;
+	
+	/* store the text following the cursor for reinsertion later */
+	if ((cursor_pos + skip_len) < len)
+		postfix = g_utf8_offset_to_pointer(text, cursor_pos + skip_len);
+
+	for (ent_start = cursor_pos; ; --ent_start)
+	{
+		if (ent_start == 0)
+			break;
+		ch = g_utf8_offset_to_pointer(text, ent_start - 1);
+		if (ch && ch[0] == ' ')
+			break;
+	}
+
+	if (ent_start == 0 && text[0] == prefs.cmdchar[0])
+	{
+		ent_start++;
+		is_cmd = 1;
+	}
+	
+	prefix_len = ent_start;
+	elen = cursor_pos - ent_start;
+
+	g_utf8_strncpy (ent, g_utf8_offset_to_pointer (text, prefix_len), elen);
+
+	is_nick = (ent[0] == '#' || ent[0] == '&' || is_cmd) ? 0 : 1;
+	
+	if (sess->type == SESS_DIALOG && is_nick)
+	{
+		/* tab in a dialog completes the other person's name */
+		if (rfc_ncasecmp (sess->channel, ent, elen) == 0)
+		{
+			result =  sess->channel;
+			is_nick = 0;
+		}
+		else
+			return 2;
+	}
+	else
+	{
+		if (is_nick)
+		{
+			gcomp = g_completion_new((GCompletionFunc)gcomp_nick_func);
+			tmp_list = userlist_double_list(sess); /* create a temp list so we can free the memory */
+		}
+		else
+		{
+			gcomp = g_completion_new (NULL);
+			if (is_cmd)
 			{
-				tab_comp_chan (sess, wid);
+				tmp_list = cmdlist_double_list (command_list);
+				for(i = 0; xc_cmds[i].name != NULL ; i++)
+				{
+					tmp_list = g_list_prepend (tmp_list, xc_cmds[i].name);
+				}
+				tmp_list = plugin_command_list(tmp_list);
+			}
+			else
+				tmp_list = chanlist_double_list (sess_list);
+		}
+		tmp_list = g_list_reverse(tmp_list); /* make the comp entries turn up in the right order */
+		g_completion_set_compare (gcomp, (GCompletionStrncmpFunc)rfc_ncasecmp);
+		if (tmp_list)
+		{
+			g_completion_add_items (gcomp, tmp_list);
+			g_list_free (tmp_list);
+		}
+
+		if (comp && !(rfc_ncasecmp(old_gcomp.data, ent, old_gcomp.elen) == 0))
+		{
+			key_action_tab_clean ();
+			comp = 0;
+		}
+	
+#if GLIB_CHECK_VERSION(2,4,0)
+		list = g_completion_complete_utf8 (gcomp, comp ? old_gcomp.data : ent, &result);
+#else
+		list = g_completion_complete (gcomp, comp ? old_gcomp.data : ent, &result);
+#endif
+		
+		if (result == NULL) /* No matches found */
+		{
+			g_completion_free(gcomp);
+			return 2;
+		}
+
+		if (comp) /* existing completion */
+		{
+			while(list) /* find the current entry */
+			{
+				if(rfc_ncasecmp(list->data, ent, elen) == 0)
+				{
+					found = 1;
+					break;
+				}
+				list = list->next;
+			}
+
+			if (found)
+			{
+				if (!(d1 && d1[0])) /* not holding down shift */
+				{
+					if (g_list_next(list) == NULL)
+						list = g_list_first(list);
+					else
+						list = g_list_next(list);
+				}
+				else
+				{
+					if (g_list_previous(list) == NULL)
+						list = g_list_last(list);
+					else
+						list = g_list_previous(list);
+				}
+				g_free(result);
+				result = (char*)list->data;
+			}
+			else
+			{
+				g_free(result);
+				g_completion_free(gcomp);
 				return 2;
 			}
 		}
-
-		/* check if we can match a command */
-		else if (chan_start == 0 && text[0] == prefs.cmdchar[0])
+		else
 		{
-			tab_comp_cmd (sess, wid);
-			return 2;
-		}
+			strcpy(old_gcomp.data, ent);
+			old_gcomp.elen = elen;
 
-		/* check if we can match a nick */
-		else if (chan_start == 0 || text[chan_start] == ' ')
-		{
-			if (d1 && d1[0])
+			/* Get the first nick and put out the data for future nickcompletes */
+			if (prefs.completion_amount && g_list_length (list) <= prefs.completion_amount)
 			{
-				if (tab_nick_comp (sess, wid, 1) == -1)
-					return 1;
+				g_free(result);
+				result = (char*)list->data;
 			}
-
 			else
 			{
-				if (tab_nick_comp (sess, wid, 0) == -1)
-					return 1;
+				/* bash style completion */
+				if (g_list_next(list) != NULL)
+				{
+					if (strlen (result) > elen) /* the largest common prefix is larger than nick, change the data */
+					{
+						if (prefix_len)
+							g_utf8_strncpy (buf, text, prefix_len);
+						strncat (buf, result, COMP_BUF - prefix_len);
+						cursor_pos = strlen (buf);
+						g_free(result);
+#if !GLIB_CHECK_VERSION(2,4,0)
+						g_utf8_validate (buf, -1, (const gchar **)&result);
+						(*result) = 0;
+#endif
+						if (postfix)
+						{
+							strcat (buf, " ");
+							strncat (buf, postfix, COMP_BUF - cursor_pos -1);
+						}
+						gtk_entry_set_text (GTK_ENTRY (t), buf);
+						gtk_editable_set_position (GTK_EDITABLE (t), g_utf8_pointer_to_offset(buf, buf + cursor_pos));
+						buf[0] = 0;
+					}
+					else
+						g_free(result);
+					while (list)
+					{
+						len = strlen (buf);	/* current buffer */
+						elen = strlen (list->data);	/* next item to add */
+						if (len + elen + 2 >= COMP_BUF) /* +2 is space + null */
+						{
+							PrintText (sess, buf);
+							buf[0] = 0;
+							len = 0;
+						}
+						strcpy (buf + len, (char *) list->data);
+						strcpy (buf + len + elen, " ");
+						list = list->next;
+					}
+					PrintText (sess, buf);
+					g_completion_free(gcomp);
+					return 2;
+				}
+				/* Only one matching entry */
+				g_free(result);
+				result = list->data;
 			}
-
-			return 2;
 		}
 	}
-
+	
+	if(result)
+	{
+		if (prefix_len)
+			g_utf8_strncpy(buf, text, prefix_len);
+		strncat (buf, result, COMP_BUF - (prefix_len + 3)); /* make sure nicksuffix and space fits */
+		if(!prefix_len && is_nick)
+			strcat (buf, &prefs.nick_suffix[0]);
+		strcat (buf, " ");
+		cursor_pos = strlen (buf);
+		if (postfix)
+			strncat (buf, postfix, COMP_BUF - cursor_pos - 2);
+		gtk_entry_set_text (GTK_ENTRY (t), buf);
+		gtk_editable_set_position (GTK_EDITABLE (t), g_utf8_pointer_to_offset(buf, buf + cursor_pos));
+	}
+	if (gcomp)
+		g_completion_free(gcomp);
 	return 2;
 }
+#undef COMP_BUF
 
 static int
 key_action_comp_chng (GtkWidget * wid, GdkEventKey * ent, char *d1, char *d2,
-							 struct session *sess)
+		struct session *sess)
 {
-	if (d1 && d1[0] != 0)
-		nick_comp_chng (sess, wid, 1);
-	else
-		nick_comp_chng (sess, wid, 0);
-
+	key_action_tab_comp(wid, ent, d1, d2, sess);
 	return 2;
 }
 
+
 static int
 key_action_replace (GtkWidget * wid, GdkEventKey * ent, char *d1, char *d2,
 						  struct session *sess)
@@ -1495,695 +1784,3 @@ replace_handle (GtkWidget *t)
 	}
 }
 
-static int
-nick_comp_get_nick (char *tx, char *n, int n_size)
-{
-	int c, len = strlen (tx);
-
-	for (c = 0; c < len && c < n_size; c++)
-	{
-      if (tx[c] == ':' || tx[c] == ',' || tx[c] == prefs.nick_suffix[0])
-		{
-			n[c] = 0;
-			return 0;
-		}
-		if (tx[c] == ' ' || tx[c] == '.' || tx[c] == 0)
-			return -1;
-		n[c] = tx[c];
-	}
-	return -1;
-}
-
-static void
-nick_comp_chng (session *sess, GtkWidget * t, int updown)
-{
-	struct User *user, *last = NULL;
-	const char *text;
-	char nick[NICKLEN], *lastnick, *newtext;
-	int len, slen;
-	GSList *head, *list;
-
-	text = gtk_entry_get_text (GTK_ENTRY (t));
-	if (nick_comp_get_nick ((char *)text, nick, sizeof (nick)) == -1)
-		return;
-
-	len = strlen (nick);
-	head = list = userlist_flat_list (sess);
-
-	while (list)
-	{
-		user = (struct User *) list->data;
-		slen = strlen (user->nick);
-		if (len != slen)
-		{
-			last = user;
-			list = list->next;
-			continue;
-		}
-		if (rfc_ncasecmp (user->nick, nick, len) == 0)
-		{
-			if (updown == 0)
-			{
-				if (list->next == NULL)
-				{
-					g_slist_free (head);
-					return;
-				}
-				user->weight--;
-				((struct User *)list->next->data)->weight++;
-				lastnick = ((struct User *)list->next->data)->nick;
-			} else
-			{
-				if (last == NULL)
-				{
-					g_slist_free (head);
-					return;
-				}
-				user->weight--;
-				last->weight++;
-				lastnick = last->nick;
-			}
-
-			newtext = (char *) malloc (strlen (text) + strlen (lastnick) + 1);
-			sprintf (newtext, "%s%s", lastnick, text + strlen (nick));
-			gtk_entry_set_text (GTK_ENTRY (t), newtext);
-			free (newtext);
-			gtk_editable_set_position (GTK_EDITABLE (t), -1);
-			g_slist_free (head);
-			return;
-
-		}
-		last = user;
-		list = list->next;
-	}
-	g_slist_free (head);
-}
-
-static void
-tab_comp_find_common (char *a, char *b)
-{
-	int c;
-	int alen = strlen (a), blen = strlen (b), len;
-
-	if (blen > alen)
-		len = alen;
-	else
-		len = blen;
-	for (c = 0; c < len; c++)
-	{
-		if (a[c] != b[c])
-		{
-			a[c] = 0;
-			return;
-		}
-	}
-	a[c] = 0;
-}
-
-static void
-tab_comp_cmd (session *sess, GtkWidget * t)
-{
-	char *last = NULL, *cmd, *postfix = NULL;
-	const char *text;
-	GSList *list = command_list;
-	struct popup *pop;
-	int len, i, slen;
-	char buf[2048];
-	char lcmd[2048];
-
-	text = gtk_entry_get_text (GTK_ENTRY (t));
-	if (text[0] == 0 || text[1] == 0)
-		return;
-	text++;
-	len = strlen (text);
-
-	for (i = 0; i < len; i++)
-	{
-		if (text[i] == ' ')
-		{
-			postfix = (char *)&text[i + 1];
-			len = i - 1;
-			break;
-		}
-	}
-
-	while (list)
-	{
-		pop = (struct popup *) list->data;
-		slen = strlen (pop->name);
-		if (len > slen)
-		{
-			list = list->next;
-			continue;
-		}
-		if (strncasecmp (pop->name, (char *)text, len) == 0)
-		{
-			if (last == NULL)
-			{
-				last = pop->name;
-				snprintf (lcmd, sizeof (lcmd), "%s", last);
-			} else if (last > (char *) 1)
-			{
-				snprintf (buf, sizeof (buf), "%s %s ", last, pop->name);
-				PrintText (sess, buf);
-				last = (void *) 1;
-				tab_comp_find_common (lcmd, pop->name);
-			} else if (last == (void *) 1)
-			{
-				PrintText (sess, pop->name);
-				tab_comp_find_common (lcmd, pop->name);
-			}
-		}
-		list = list->next;
-	}
-
-	i = 0;
-	while (xc_cmds[i].name != NULL)
-	{
-		cmd = xc_cmds[i].name;
-		slen = strlen (cmd);
-		if (len > slen)
-		{
-			i++;
-			continue;
-		}
-		if (strncasecmp (cmd, (char *)text, len) == 0)
-		{
-			if (last == NULL)
-			{
-				last = cmd;
-				snprintf (lcmd, sizeof (lcmd), "%s", last);
-			} else if (last > (char *) 1)
-			{
-				snprintf (buf, sizeof (buf), "%s %s ", last, cmd);
-				PrintText (sess, buf);
-				last = (void *) 1;
-				tab_comp_find_common (lcmd, cmd);
-			} else if (last == (void *) 1)
-			{
-				PrintText (sess, cmd);
-				tab_comp_find_common (lcmd, cmd);
-			}
-		}
-		i++;
-	}
-
-	if (last == NULL)
-		return;
-	if (last == (void *) 1)
-		PrintText (sess, "\n");
-
-	if (last > (char *) 1)
-	{
-		if (strlen (last) > (sizeof (buf) - 1))
-			return;
-		if (postfix == NULL)
-			snprintf (buf, sizeof (buf), "%c%s ", prefs.cmdchar[0], last);
-		else
-			snprintf (buf, sizeof (buf), "%c%s %s", prefs.cmdchar[0], last, postfix);
-		gtk_entry_set_text (GTK_ENTRY (t), buf);
-		gtk_editable_set_position (GTK_EDITABLE (t), -1);
-		return;
-	} else if (strlen (lcmd) > (sizeof (buf) - 1))
-		return;
-	if (postfix == NULL)
-		snprintf (buf, sizeof (buf), "%c%s", prefs.cmdchar[0], lcmd);
-	else
-		snprintf (buf, sizeof (buf), "%c%s %s", prefs.cmdchar[0], lcmd, postfix);
-	gtk_entry_set_text (GTK_ENTRY (t), buf);
-	gtk_editable_set_position (GTK_EDITABLE (t), -1);
-}
-
-/* handles tab completion for channel names */
-static void
-tab_comp_chan (session *sess, GtkWidget * t)
-{
-	char buf[2048], choices[2048], lchan[2048];
-	char *chan, *last = NULL, *postfix = NULL;
-	char exact_match = 0;
-	const char *text;
-	int chan_start, choices_pos = 0, len, num_matches = 0, prefix_len, slen;
-	gint cursor_pos;
-	GSList *list, *sorted_chanlist = NULL;
-	session *tmp_sess;
-
-	/* obtain the entered text and cursor position */
-	text = gtk_entry_get_text (GTK_ENTRY (t));
-	if (text[0] == 0)
-		return;
-
-	cursor_pos = gtk_editable_get_position (GTK_EDITABLE (t));
-
-	/* store the text following the cursor for reinsertion later */
-	if (cursor_pos < strlen(text))
-		postfix = (char *)&text[cursor_pos];
-
-	/* search backwards to find the # (we are guaranteed to find it */
-	for (chan_start = cursor_pos; ; --chan_start)
-	{
-		if (text[chan_start] == '#')
-		{
-			if (chan_start == 0 || text[chan_start - 1] == ' ')
-				break;
-		}
-	}
-
-	/* store the amount of text preceding the cursor for reinsertion later */
-	prefix_len = chan_start;
-
-	/* form an alphabetically sorted list of the channel names */
-	for (list = sess_list; list; list = list->next)
-	{
-		tmp_sess = list->data;
-
-		if (tmp_sess->type == SESS_CHANNEL)
-		{
-			if (tmp_sess->channel[0] != '\0' && g_slist_find_custom (sorted_chanlist, tmp_sess->channel, alpha_string_compare) == NULL)
-				sorted_chanlist = g_slist_insert_sorted (sorted_chanlist, tmp_sess->channel, alpha_string_compare);
-		}
-	}
-
-	/* completing "#" displays the first channel */
-	list = sorted_chanlist;
-
-	if (chan_start == cursor_pos - 1)
-	{
-		/* if we are in no channels do nothing */
-		if (!list)
-			return;
-
-		else
-			safe_strcpy (lchan, list->data, sizeof (lchan));
-	}
-
-	else
-	{
-		/* find candidates for partial channel name completion */
-		len = cursor_pos - chan_start;
-
-		while (list)
-		{
-			/* quick check on string lengths to eliminate short channel names */
-			chan = (char *)list->data;
-			slen = strlen (chan);
-
-			if (len > slen)
-			{
-				list = list->next;
-				continue;
-			}
-
-			/* check if the entered text matches the channel name so far */
-			if (strncasecmp (chan, (char *)&text[chan_start], len) == 0)
-			{
-				++ num_matches;
-
-				/* if the channel matches exactly, and we've not found another
-				   matching channel, remember it */
-				if (exact_match == 0 && len == slen )
-					exact_match = 1;
-
-				/* if we had an exact matching channel, but have now found a
-				   longer matching one, forget the exact match */
-				else if (exact_match == 1 && len < slen)
-					exact_match = 2;
-
-				/* add the channel name to our list of potential matches */
-				if (choices_pos + strlen (chan) + 1 >= sizeof (choices))
-				{
-					PrintText (sess, choices);
-					choices_pos = 0;
-				}
-
-				snprintf (&choices[choices_pos], sizeof (choices) - choices_pos, "%s ", chan);
-				choices_pos += strlen (chan) + 1;
-
-				/* store the most matching amount of the channel name so far */
-				if (last == NULL)
-					safe_strcpy (lchan, chan, sizeof (lchan));
-
-				last = chan;
-				tab_comp_find_common (lchan, last);
-			}
-
-			list = list->next;
-		}
-
-		/* do nothing if no matches were found */
-
-		if (num_matches == 0)
-		{
-			if (sorted_chanlist)
-				g_slist_free (sorted_chanlist);
-
-			return;
-		}
-
-		/* on tabbing an already complete channel, cycle to the next */
-		if (exact_match)
-		{
-			GSList *this_chan = g_slist_find_custom (sorted_chanlist, lchan, alpha_string_compare);
-
-			if (g_slist_next (this_chan) || exact_match == 2)
-				this_chan = g_slist_next (this_chan);
-
-			else
-				this_chan = sorted_chanlist;
-
-			safe_strcpy (lchan, this_chan->data, sizeof (lchan));
-		}
-
-		/* display the potential channel names if appropriate */
-		if (num_matches > 1 && exact_match != 2)
-			PrintText (sess, choices);
-	}
-
-	/* build the completed channel name with preceding and succeeding text */
-	safe_strcpy (buf, text, sizeof (buf));
-
-	if (postfix == NULL)
-		snprintf (&buf[prefix_len], sizeof (buf) - prefix_len - 1, "%s", lchan);
-	else
-		snprintf (&buf[prefix_len], sizeof (buf) - prefix_len - strlen (postfix) - 1, "%s%s", lchan, postfix);
-
-	/* update the text entry widget */
-	gtk_entry_set_text (GTK_ENTRY (t), buf);
-	gtk_editable_set_position (GTK_EDITABLE (t), prefix_len + strlen (lchan));
-
-	/* free the list we allocated */
-	g_slist_free (sorted_chanlist);
-}
-
-/* a sort comparison function for alphabetical ordering */
-
-static gint alpha_string_compare (gconstpointer str1, gconstpointer str2)
-{
-	return (strcmp (str1, str2));
-}
-
-/* In the following 'b4' is *before* the text (just say b4 and before out loud)
-   and c5 is *after* (because c5 is next after b4, get it??) --AGL */
-
-static int
-tab_nick_comp_next (struct session *sess, GtkWidget * wid, char *b4,
-						  char *nick, char *c5, int shift)
-{
-	struct User *user = 0, *last = NULL;
-	int pos;
-	char buf[4096];
-	GSList *head, *list;
-
-	head = list = userlist_flat_list (sess);
-	while (list)
-	{
-		user = (struct User *) list->data;
-		if (strcmp (user->nick, nick) == 0)
-			break;
-		last = user;
-		list = list->next;
-	}
-
-	if (!list)
-	{
-		g_slist_free (head);
-		return 0;
-	}
-
-	if (b4[0] != 0)
-		strcat (b4, " ");
-
-	if (shift)
-	{
-		if (last)
-			nick = last->nick;
-		else
-		{
-			/* making this consistent with the (!shift) behaviour */
-			list = g_slist_last (head);
-			if (list)
-				nick = ((struct User *)list->data)->nick;
-		}
-	} else
-	{
-		if (list->next)
-			nick = ((struct User *) list->next->data)->nick;
-		else
-		{
-			if (head)
-				nick = ((struct User *) head->data)->nick;
-			/*else
-				nick = nick;*/
-		}
-	}
-
-	snprintf (buf, sizeof (buf), "%s%s%s", b4, nick, c5);
-
-	pos = strlen (buf) - strlen (c5);
-	gtk_entry_set_text (GTK_ENTRY (wid), buf);
-	gtk_editable_set_position (GTK_EDITABLE (wid), pos);
-	g_slist_free (head);
-
-	return 1;
-}
-
-static int
-tab_nick_comp (session *sess, GtkWidget *t, int shift)
-{
-	struct User *user, *match_user = NULL;
-	const char *origtext;
-	char *text, not_nick_chars[5] = { ' ', '.', '?', 0, 0};
-	int len, slen, first, i, cursor_pos, match_count = 0, match_pos = 0;
-	char buf[2048], nick_buf[2048], *b4 = NULL, *c5 = NULL;
-	char *match_text = NULL, *current_nick = NULL;
-	unsigned char match_char = 0xff;
-	GSList *head = NULL, *list, *match_list = NULL, *first_match = NULL;
-	int ret = 0;	/* return value */
-
-	origtext = gtk_entry_get_text (GTK_ENTRY (t));
-
-	/* make a copy, since we use this buffer */
-	text = strdup (origtext);
-	len = strlen (text);
-
-	/* Is the text more than just a nick? */
-	not_nick_chars[3] = prefs.nick_suffix[0];
-
-	if (strcspn (text, not_nick_chars) != len)
-	{
-		/* If we're doing old-style nick completion and the text input widget
-		 * contains a string of the format: "nicknameSUFFIX" or "nicknameSUFFIX ",
-		 * where SUFFIX is the Nickname Completion Suffix character, then cycle
-		 * through the available nicknames.
-		 */
-		if (prefs.old_nickcompletion)
-		{
-			char *space = strchr (text, ' ');
-
-			if ((!space || space == &text[len - 1]) &&
-				 text[len - (space ? 2 : 1)] == prefs.nick_suffix[0])
-			{
-				/* This causes the nickname to cycle. */
-				nick_comp_chng (sess, t, shift);
-				goto compdone1; /* free(text) and return 0; */
-			}
-		}
-
-		cursor_pos = gtk_editable_get_position (GTK_EDITABLE (t));
-
-		/* !! FIXME !! */
-		if (len - cursor_pos < 0)
-			goto compdone1;	/* free(text) and return 0; */
-
-		b4 = (char *) malloc (len + 2);	/* 1 extra for the strcat */
-		c5 = (char *) malloc (len + 1);
-
-		/* this's added for some sanity. or else, nick replacements are just insane. */
-
-		while (text[cursor_pos])
-		{
-			if (text[cursor_pos] == ' ')
-				break;
-			cursor_pos++;
-		}
-
-		memmove (c5, text + cursor_pos, len - cursor_pos);
-		c5[len - cursor_pos] = 0;
-		memcpy (b4, text, len + 1);
-
-		for (i = cursor_pos - 1; i >= 0; i--)
-		{
-			if (b4[i] == ' ')
-			{
-				b4[i] = 0;
-				break;
-			}
-			b4[i] = 0;
-		}
-
-		memmove (text, text + i + 1, cursor_pos - i - 1);
-		text[cursor_pos - i - 1] = 0;
-
-		if (tab_nick_comp_next (sess, t, b4, text, c5, shift))
-		{
-			ret = 1;
-			goto compdone;
-		}
-
-		first = 0;
-
-	} else
-	{
-		first = 1;
-	}
-
-	if (text[0] == 0)
-	{
-		ret = -1;
-		goto compdone;
-	}
-
-	len = strlen (text);
-	head = list = userlist_flat_list (sess);
-
-	if (sess->type == SESS_DIALOG)
-	{
-		/* tab in a dialog completes the other person's name */
-		if (rfc_ncasecmp (sess->channel, text, len) == 0)
-			match_list = g_slist_prepend (match_list, sess->channel);
-	}
-
-	else
-	{
-		/* make a list of matches */
-		while (list)
-		{
-			user = (struct User *) list->data;
-			slen = strlen (user->nick);
-			if (len > slen)
-			{
-				list = list->next;
-				continue;
-			}
-			if (rfc_ncasecmp (user->nick, text, len) == 0)
-				match_list = g_slist_prepend (match_list, user->nick);
-			list = list->next;
-		}
-	}
-
-	match_list = g_slist_reverse (match_list); /* faster then _append */
-	match_count = g_slist_length (match_list);
-
-	/* no matches, return */
-	if (match_count == 0)
-		goto compdone;
-
-	first_match = match_list;
-	match_pos = len;
-
-	/* if we have more then 1 match, we want to act like readline and grab common chars */
-	if ((!prefs.old_nickcompletion) && (match_count > 1))
-	{
-		while (1)
-		{
-			while (match_list)
-			{
-				current_nick = match_list->data;
-				if (match_char == 0xff)
-				{
-					match_char = current_nick[match_pos];
-				} else if (rfc_tolower (current_nick[match_pos]) != rfc_tolower (match_char))
-				{
-					match_text = malloc (match_pos + 1);
-					memcpy (match_text, current_nick, match_pos);
-					match_text[match_pos] = 0;
-					match_pos = -1;
-					break;
-				}
-				match_list = match_list->next;
-			}
-
-			if (match_pos == -1)
-				break;
-
-			match_list = first_match;
-			match_char = 0xff;
-			++match_pos;
-		}
-
-		match_list = first_match;
-	} else
-		match_user = (struct User *) match_list->data;
-	
-	/* no match, if we found more common chars among matches, display them in entry */
-	if (match_user == NULL)
-	{
-		nick_buf[0] = 0;
-		while (match_list)
-		{
-			if (strlen (nick_buf) + strlen (match_list->data) >= sizeof (nick_buf))
-			{
-				PrintText (sess, nick_buf);
-				nick_buf[0] = 0;		
-			}
-			sprintf (nick_buf, "%s%s ", nick_buf, (char *)match_list->data);
-			match_list = match_list->next;
-		}
-		PrintText (sess, nick_buf);
-
-		if (first)
-		{
-			snprintf (buf, sizeof (buf), "%s", match_text);
-			cursor_pos = strlen (buf);
-		}
-		else
-		{
-			if (b4[0] != 0)
-				strcat (b4, " ");
-			snprintf (buf, sizeof (buf), "%s%s%s", b4, match_text, c5);
-			cursor_pos = strlen (buf) - strlen (c5);
-		}
-		free (match_text);
-	} else
-	{
-		if (first)
-		{
-			snprintf (buf, sizeof (buf), "%s%c ", match_user->nick,
-						 prefs.nick_suffix[0]);
-			cursor_pos = strlen (buf);
-		}
-		else
-		{
-			if (b4[0] != 0)
-				strcat (b4, " ");
-			snprintf (buf, sizeof (buf), "%s%s%s", b4, match_user->nick, c5);
-			cursor_pos = strlen (buf) - strlen (c5);
-		}
-	}
-
-	{
-		/* This fixes invalid UTF-8 sequence by completion of
-		   a multibyte nick. -- FIXME: only a bandaid solution -- */
-		char *valid_end;
-		g_utf8_validate (buf, -1, (const gchar **)&valid_end);
-		(*valid_end) = 0;
-	}
-
-	gtk_entry_set_text (GTK_ENTRY (t), buf);
-	/* GeEkMaN: Restore the last cursor position */
-	gtk_editable_set_position (GTK_EDITABLE (t), cursor_pos);
-
-compdone:
-	if (match_list)
-		g_slist_free (match_list);
-	if (b4)
-		free (b4);
-	if (c5)
-		free (c5);
-
-compdone1:
-	free (text);
-	if (head)
-		g_slist_free (head);
-
-	return ret;
-}
