@@ -47,7 +47,9 @@
 #include "../common/text.h"
 #include "../common/xchatc.h"
 #include "../common/outbound.h"
+#include "../common/inbound.h"
 #include "../common/plugin.h"
+#include "../common/modes.h"
 #include "fe-gtk.h"
 #include "banlist.h"
 #include "gtkutil.h"
@@ -60,7 +62,7 @@
 #include "xtext.h"
 
 static void mg_create_entry (session *sess, GtkWidget *box);
-static void mg_link_irctab (session *sess);
+static void mg_link_irctab (session *sess, int focus);
 
 static session_gui static_mg_gui;
 static session_gui *mg_gui = NULL;	/* the shared irc tab */
@@ -72,9 +74,9 @@ GtkWidget *parent_window = NULL;			/* the master window */
 
 GtkStyle *input_style;
 
-PangoAttrList *newdata_list;
-PangoAttrList *nickseen_list;
-PangoAttrList *newmsg_list = NULL;
+static PangoAttrList *newdata_list;
+static PangoAttrList *nickseen_list;
+static PangoAttrList *newmsg_list;
 static PangoAttrList *plain_list = NULL;
 
 
@@ -84,12 +86,27 @@ mg_attr_list_create (GdkColor *col)
 	PangoAttribute *attr;
 	PangoAttrList *list;
 
-	attr = pango_attr_foreground_new (col->red, col->green, col->blue);
-	attr->start_index = 0;
-	attr->end_index = 0xffff;
-
 	list = pango_attr_list_new ();
-	pango_attr_list_insert (list, attr);
+
+	if (col)
+	{
+		attr = pango_attr_foreground_new (col->red, col->green, col->blue);
+		attr->start_index = 0;
+		attr->end_index = 0xffff;
+		pango_attr_list_insert (list, attr);
+	}
+
+	if (prefs.tab_small)
+	{
+#ifdef WIN32
+		attr = pango_attr_scale_new (PANGO_SCALE_SMALL);
+#else
+		attr = pango_attr_scale_new (PANGO_SCALE_X_SMALL);
+#endif
+		attr->start_index = 0;
+		attr->end_index = 0xffff;
+		pango_attr_list_insert (list, attr);
+	}
 
 	return list;
 }
@@ -97,19 +114,18 @@ mg_attr_list_create (GdkColor *col)
 static void
 mg_create_tab_colors (void)
 {
-	if (newmsg_list)
+	if (plain_list)
 	{
+		pango_attr_list_unref (plain_list);
 		pango_attr_list_unref (newmsg_list);
 		pango_attr_list_unref (newdata_list);
 		pango_attr_list_unref (nickseen_list);
 	}
 
-	newdata_list = mg_attr_list_create (&colors[20]);
-	nickseen_list = mg_attr_list_create (&colors[21]);
-	newmsg_list = mg_attr_list_create (&colors[22]);
-
-	if (!plain_list)
-		plain_list = pango_attr_list_new ();
+	plain_list = mg_attr_list_create (NULL);
+	newdata_list = mg_attr_list_create (&colors[COL_NEW_DATA]);
+	nickseen_list = mg_attr_list_create (&colors[COL_HILIGHT]);
+	newmsg_list = mg_attr_list_create (&colors[COL_NEW_MSG]);
 }
 
 #ifdef WIN32
@@ -148,6 +164,38 @@ flash_window (GtkWidget *win)
 	flash (&fi);
 	/*FlashWindowEx (&fi);*/
 }
+#else
+
+#ifdef USE_XLIB
+#include <gdk/gdkx.h>
+
+static void
+set_window_urgency (GtkWidget *win, gboolean set)
+{
+	XWMHints *hints;
+
+	hints = XGetWMHints(GDK_WINDOW_XDISPLAY(win->window), GDK_WINDOW_XWINDOW(win->window));
+	if (set)
+		hints->flags |= XUrgencyHint;
+	else
+		hints->flags &= ~XUrgencyHint;
+	XSetWMHints(GDK_WINDOW_XDISPLAY(win->window),
+	            GDK_WINDOW_XWINDOW(win->window), hints);
+	XFree(hints);
+}
+
+static void
+flash_window (GtkWidget *win)
+{
+	set_window_urgency (win, TRUE);
+}
+
+static void
+unflash_window (GtkWidget *win)
+{
+	set_window_urgency (win, FALSE);
+}
+#endif
 #endif
 
 /* set a tab plain, red, light-red, or blue */
@@ -186,8 +234,8 @@ fe_set_tab_color (struct session *sess, int col, int flash)
 		}
 	}
 
-#ifdef WIN32
-	if (flash)
+#if defined(WIN32) || defined(USE_XLIB)
+	if (flash && prefs.flash_hilight)
 		flash_window (sess->gui->window);
 #endif
 }
@@ -298,8 +346,8 @@ fe_set_title (session *sess)
 	switch (type)
 	{
 	case SESS_DIALOG:
-		snprintf (tbuf, sizeof (tbuf), "X-Chat ["VERSION"]: Dialog with %s @ %s",
-					 sess->channel, sess->server->servername);
+		snprintf (tbuf, sizeof (tbuf), "X-Chat ["VERSION"]: %s %s @ %s",
+					 _("Dialog with"), sess->channel, sess->server->servername);
 		break;
 	case SESS_SERVER:
 		snprintf (tbuf, sizeof (tbuf), "X-Chat ["VERSION"]: %s @ %s",
@@ -325,6 +373,16 @@ fe_set_title (session *sess)
 	gtk_window_set_title (GTK_WINDOW (sess->gui->window), tbuf);
 }
 
+static gboolean
+mg_windowstate_cb (GtkWidget *wid, GdkEvent *event, gpointer userdata)
+{
+	prefs.gui_win_state = 0;
+	if (gdk_window_get_state (wid->window) & GDK_WINDOW_STATE_MAXIMIZED)
+		prefs.gui_win_state = 1;
+
+	return FALSE;
+}
+
 static gboolean
 mg_configure_cb (GtkWidget *wid, GdkEventConfigure *event, session *sess)
 {
@@ -332,18 +390,13 @@ mg_configure_cb (GtkWidget *wid, GdkEventConfigure *event, session *sess)
 	{
 		if (mg_gui)
 		{
-/*			g_idle_add ((GSourceFunc)tab_group_resize, mg_gui->tabs_box);*/
 			if (prefs.mainwindow_save)
 			{
 				sess = current_sess;
 				gtk_window_get_position (GTK_WINDOW (wid), &prefs.mainwindow_left,
 												 &prefs.mainwindow_top);
 				gtk_window_get_size (GTK_WINDOW (wid), &prefs.mainwindow_width,
-												 &prefs.mainwindow_height);
-				/*prefs.mainwindow_left = event->x;
-				prefs.mainwindow_top = event->y;
-				prefs.mainwindow_width = event->width;
-				prefs.mainwindow_height = event->height;*/
+											&prefs.mainwindow_height);
 			}
 		}
 	}
@@ -356,10 +409,6 @@ mg_configure_cb (GtkWidget *wid, GdkEventConfigure *event, session *sess)
 											 &prefs.dialog_top);
 			gtk_window_get_size (GTK_WINDOW (wid), &prefs.dialog_width,
 										&prefs.dialog_height);
-			/*prefs.dialog_left = event->x;
-			prefs.dialog_top = event->y;
-			prefs.dialog_width = event->width;
-			prefs.dialog_height = event->height;*/
 		}
 
 		if (((GtkXText *) sess->gui->xtext)->transparent)
@@ -404,7 +453,7 @@ mg_focus (session *sess)
 	/* dirty trick to avoid auto-selection */
 	GTK_ENTRY (sess->gui->input_box)->editable = 0;
 	gtk_widget_grab_focus (sess->gui->input_box);
-	GTK_ENTRY (sess->gui->input_box)->editable = 1;
+	gtk_editable_set_editable (GTK_EDITABLE (sess->gui->input_box), TRUE);
 
 	sess->server->front_session = sess;
 
@@ -564,6 +613,8 @@ mg_set_topic_tip (session *sess)
 		if (GTK_ENTRY (sess->gui->topic_entry)->text &&
 			 GTK_ENTRY (sess->gui->topic_entry)->text[0])
 			add_tip (sess->gui->topic_entry, GTK_ENTRY (sess->gui->topic_entry)->text);
+		else
+			add_tip (sess->gui->topic_entry, ""); /* hSP: set topic to empty in case it isn't set.. */
 	}
 }
 
@@ -615,6 +666,8 @@ mg_populate (session *sess)
 		gtk_widget_hide (gui->topicbutton_box);
 		/* hide the userlist */
 		mg_userlist_showhide (sess, FALSE);
+		/* shouldn't edit the topic */
+		gtk_editable_set_editable (GTK_EDITABLE (gui->topic_entry), FALSE);
 		break;
 	case SESS_SERVER:
 		if (prefs.chanmodebuttons)
@@ -623,6 +676,8 @@ mg_populate (session *sess)
 		gtk_widget_hide (gui->dialogbutton_box);
 		/* hide the userlist */
 		mg_userlist_showhide (sess, FALSE);
+		/* shouldn't edit the topic */
+		gtk_editable_set_editable (GTK_EDITABLE (gui->topic_entry), FALSE);
 		break;
 	default:
 		/* hide the dialog buttons */
@@ -631,6 +686,8 @@ mg_populate (session *sess)
 			gtk_widget_show (gui->topicbutton_box);
 		/* show the userlist */
 		mg_userlist_showhide (sess, TRUE);
+		/* let the topic be editted */
+		gtk_editable_set_editable (GTK_EDITABLE (gui->topic_entry), TRUE);
 	}
 
 	/* move to THE irc tab */
@@ -885,12 +942,7 @@ mg_create_sess_tree (GtkWidget *menu)
 		if (serv->servername[0] == 0)
 			top_item = gtk_menu_item_new_with_label (_("<none>"));
 		else
-		{
-			if (serv->networkname)
-				top_item = gtk_menu_item_new_with_label (serv->networkname);
-			else
-				top_item = gtk_menu_item_new_with_label (serv->servername);
-		}
+			top_item = gtk_menu_item_new_with_label (server_get_network (serv, TRUE));
 
 		gtk_menu_shell_prepend (GTK_MENU_SHELL (menu), top_item);
 		gtk_widget_show (top_item);
@@ -971,7 +1023,7 @@ mg_close_sess (session *sess)
 static void
 mg_detach_tab_cb (GtkWidget *item, session *sess)
 {
-	mg_link_irctab (sess);
+	mg_link_irctab (sess, 0);
 }
 
 static void
@@ -991,7 +1043,7 @@ mg_color_insert (GtkWidget *item, gpointer userdata)
 {
 	char buf[32];
 
-	sprintf (buf, "%%C%02d", GPOINTER_TO_INT (userdata));
+	sprintf (buf, "\003%02d", GPOINTER_TO_INT (userdata));
 	key_action_insert (current_sess->gui->input_box, 0, buf, 0, 0);
 }
 
@@ -1045,17 +1097,6 @@ mg_tab_press_cb (GtkWidget *wid, GdkEventButton *event, session *sess)
 
 	menu = gtk_menu_new ();
 
-	mg_create_icon_item (_("Close Tab"), GTK_STOCK_CLOSE, menu,
-								mg_destroy_tab_cb, wid);
-	if (sess)
-		mg_create_icon_item (_("Detach Tab"), GTK_STOCK_REDO, menu,
-									mg_detach_tab_cb, sess);
-
-	/* separator */
-	item = gtk_menu_item_new ();
-	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
-	gtk_widget_show (item);
-
 	if (sess)
 	{
 		item = gtk_menu_item_new_with_label (sess->channel[0] ? sess->channel : _("<none>"));
@@ -1086,6 +1127,17 @@ mg_tab_press_cb (GtkWidget *wid, GdkEventButton *event, session *sess)
 	gtk_menu_item_set_submenu (GTK_MENU_ITEM (item), submenu);
 	gtk_widget_show (item);
 
+	/* separator */
+	item = gtk_menu_item_new ();
+	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
+	gtk_widget_show (item);
+
+	mg_create_icon_item (_("Close Tab"), GTK_STOCK_CLOSE, menu,
+								mg_destroy_tab_cb, wid);
+	if (sess)
+		mg_create_icon_item (_("Detach Tab"), GTK_STOCK_REDO, menu,
+									mg_detach_tab_cb, sess);
+
 	if (sess && tabmenu_list)
 		menu_create (menu, tabmenu_list, sess->channel, FALSE);
 
@@ -1095,12 +1147,61 @@ mg_tab_press_cb (GtkWidget *wid, GdkEventButton *event, session *sess)
 	return TRUE;
 }
 
+void
+mg_dnd_drop_file (session *sess, char *target, char *uri)
+{
+	char *p, *data, *next, *fname;
+
+	p = data = strdup (uri);
+	while (*p)
+	{
+		next = strchr (p, '\r');
+		if (strncasecmp ("file:", p, 5) == 0)
+		{
+			if (next)
+				*next = 0;
+			fname = g_filename_from_uri (p, NULL, NULL);
+			if (fname)
+			{
+				/* dcc_send() expects utf-8 */
+				p = g_filename_to_utf8 (fname, -1, 0, 0, 0);
+				if (p)
+				{
+					dcc_send (sess, target, p, prefs.dcc_max_send_cps, 0);
+					g_free (p);
+				}
+				g_free (fname);
+			}
+		}
+		if (!next)
+			break;
+		p = next + 1;
+		if (*p == '\n')
+			p++;
+	}
+	free (data);
+
+}
+
+static void
+mg_dialog_dnd_drop (GtkWidget * widget, GdkDragContext * context, gint x,
+						  gint y, GtkSelectionData * selection_data, guint info,
+						  guint32 time, session *sess)
+{
+	/* sess->channel is really the nickname of dialogs */
+	mg_dnd_drop_file (sess, sess->channel, selection_data->data);
+}
+
 /* add a tabbed channel */
 
 static void
 mg_add_chan (session *sess)
 {
 	char *name = _("<none>");
+	static const GtkTargetEntry dnd_targets[] =
+	{
+		{"text/uri-list", 0, 1}
+	};
 
 	if (sess->channel[0])
 		name = sess->channel;
@@ -1108,11 +1209,12 @@ mg_add_chan (session *sess)
 	sess->res->tab = tab_group_add (sess->gui->tabs_box, name, sess->server,
 											  sess, mg_tab_press_cb, mg_link_cb,
 											  prefs.truncchans, prefs.tab_dnd);
-	g_object_set_data (G_OBJECT (sess->res->tab), "sess", sess);
-
-	if (newmsg_list == NULL)
+	if (plain_list == NULL)
 		mg_create_tab_colors ();
 
+	tab_set_attrlist (sess->res->tab, plain_list);
+	g_object_set_data (G_OBJECT (sess->res->tab), "sess", sess);
+
 	g_signal_connect (G_OBJECT (sess->res->tab), "destroy",
 					      G_CALLBACK (mg_tabdestroy_cb), sess);
 
@@ -1122,6 +1224,16 @@ mg_add_chan (session *sess)
 		gtk_xtext_set_time_stamp (sess->res->buffer, prefs.timestamp);
 		sess->res->user_model = userlist_create_model ();
 	}
+
+#ifndef WIN32	/* needs more work */
+	if (sess->type == SESS_DIALOG)
+	{
+		gtk_drag_dest_set (sess->gui->xtext, GTK_DEST_DEFAULT_ALL, dnd_targets,
+								 1, GDK_ACTION_MOVE | GDK_ACTION_COPY | GDK_ACTION_LINK);
+		g_signal_connect (G_OBJECT (sess->gui->xtext), "drag_data_received",
+								G_CALLBACK (mg_dialog_dnd_drop), sess);
+	}
+#endif
 }
 
 static void
@@ -1225,7 +1337,7 @@ mg_topic_cb (GtkWidget *entry, gpointer userdata)
 	session *sess = current_sess;
 	char *text;
 
-	if (sess->channel[0] && sess->server->connected)
+	if (sess->channel[0] && sess->server->connected && sess->type == SESS_CHANNEL)
 	{
 		text = GTK_ENTRY (entry)->text;
 		if (text[0] == 0)
@@ -1337,12 +1449,12 @@ mg_changui_destroy (session *sess)
 }
 
 static void
-mg_link_irctab (session *sess)
+mg_link_irctab (session *sess, int focus)
 {
 	if (sess->gui->is_tab)
 	{
 		mg_changui_destroy (sess);
-		mg_changui_new (sess, sess->res, 0);
+		mg_changui_new (sess, sess->res, 0, focus);
 		mg_populate (sess);
 		xchat_is_quitting = FALSE;
 		return;
@@ -1350,7 +1462,7 @@ mg_link_irctab (session *sess)
 
 	mg_unpopulate (sess);
 	mg_changui_destroy (sess);
-	mg_changui_new (sess, sess->res, 1);
+	mg_changui_new (sess, sess->res, 1, focus);
 }
 
 static void
@@ -1373,7 +1485,7 @@ mg_link_cb (GtkWidget *but, gpointer userdata)
 		return;
 	}
 
-	mg_link_irctab (current_sess);
+	mg_link_irctab (current_sess, 0);
 }
 
 static int
@@ -1402,7 +1514,7 @@ mg_change_flag (GtkWidget * wid, session *sess, char flag)
 			mode[0] = '+';
 		else
 			mode[0] = '-';
-		serv->p_chan_mode (serv, sess->channel, mode);
+		serv->p_mode (serv, sess->channel, mode);
 		serv->p_join_info (serv, sess->channel);
 		sess->ignore_mode = TRUE;
 		sess->ignore_date = TRUE;
@@ -1429,7 +1541,7 @@ flagl_hit (GtkWidget * wid, struct session *sess)
 				return;
 			}
 			snprintf (modes, sizeof (modes), "+l %d", atoi (limit_str));
-			serv->p_chan_mode (serv, sess->channel, modes);
+			serv->p_mode (serv, sess->channel, modes);
 			serv->p_join_info (serv, sess->channel);
 		}
 	} else
@@ -1450,7 +1562,7 @@ flagk_hit (GtkWidget * wid, struct session *sess)
 		if (GTK_TOGGLE_BUTTON (wid)->active)
 			modes[0] = '+';
 
-		serv->p_chan_mode (serv, sess->channel, modes);
+		serv->p_mode (serv, sess->channel, modes);
 	}
 }
 
@@ -1512,7 +1624,7 @@ mg_key_entry_cb (GtkWidget * igad, gpointer userdata)
 	{
 		snprintf (modes, sizeof (modes), "+k %s",
 				gtk_entry_get_text (GTK_ENTRY (igad)));
-		serv->p_chan_mode (serv, sess->channel, modes);
+		serv->p_mode (serv, sess->channel, modes);
 		serv->p_join_info (serv, sess->channel);
 	}
 }
@@ -1535,7 +1647,7 @@ mg_limit_entry_cb (GtkWidget * igad, gpointer userdata)
 		}
 		snprintf (modes, sizeof(modes), "+l %d", 
 				atoi (gtk_entry_get_text (GTK_ENTRY (igad))));
-		serv->p_chan_mode (serv, sess->channel, modes);
+		serv->p_mode (serv, sess->channel, modes);
 		serv->p_join_info (serv, sess->channel);
 	}
 }
@@ -1543,8 +1655,8 @@ mg_limit_entry_cb (GtkWidget * igad, gpointer userdata)
 static void
 mg_apply_entry_style (GtkWidget *entry)
 {
-	gtk_widget_modify_base (entry, GTK_STATE_NORMAL, &colors[19]);
-	gtk_widget_modify_text (entry, GTK_STATE_NORMAL, &colors[18]);
+	gtk_widget_modify_base (entry, GTK_STATE_NORMAL, &colors[COL_BG]);
+	gtk_widget_modify_text (entry, GTK_STATE_NORMAL, &colors[COL_FG]);
 	gtk_widget_modify_font (entry, input_style->font_desc);
 }
 
@@ -1618,7 +1730,7 @@ mg_dialog_button_cb (GtkWidget *wid, char *cmd)
 	/* dirty trick to avoid auto-selection */
 	GTK_ENTRY (current_sess->gui->input_box)->editable = 0;
 	gtk_widget_grab_focus (current_sess->gui->input_box);
-	GTK_ENTRY (current_sess->gui->input_box)->editable = 1;
+	gtk_editable_set_editable (GTK_EDITABLE (current_sess->gui->input_box), TRUE);
 }
 
 static void
@@ -1654,7 +1766,7 @@ mg_create_topicbar (session *sess, GtkWidget *box, char *name)
 	GtkWidget *hbox, *topic, *bbox;
 	session_gui *gui = sess->gui;
 
-	gui->topic_bar = hbox = gtk_hbox_new (FALSE, 0);
+	gui->topic_bar = hbox = gtk_hbox_new (FALSE, 1);
 	gtk_box_pack_start (GTK_BOX (box), hbox, 0, 0, 0);
 
 	mg_create_link_buttons (hbox, NULL);
@@ -1718,7 +1830,7 @@ mg_word_clicked (GtkWidget *xtext, char *word, GdkEventButton *even,
 			{
 			case WORD_URL:
 			case WORD_HOST:
-				goto_url (word);
+				fe_open_url (word);
 			}
 		}
 		return;
@@ -1728,6 +1840,8 @@ mg_word_clicked (GtkWidget *xtext, char *word, GdkEventButton *even,
 	{
 		if (sess->type == SESS_DIALOG)
 			menu_middlemenu (sess, even);
+		else if (even->type == GDK_2BUTTON_PRESS)
+			userlist_select (sess, word);
 		return;
 	}
 
@@ -1745,7 +1859,7 @@ mg_word_clicked (GtkWidget *xtext, char *word, GdkEventButton *even,
 			word+1 : word, FALSE);
 		break;
 	case WORD_CHANNEL:
-		if (*word == '@' || *word == '+')
+		if (*word == '@' || *word == '+' || *word=='^' || *word=='%' || *word=='*')
 			word++;
 		menu_chanmenu (sess, even, word);
 		break;
@@ -1775,6 +1889,7 @@ mg_update_xtext (GtkWidget *wid)
 	gtk_xtext_set_tint (xtext, prefs.tint_red, prefs.tint_green, prefs.tint_blue);
 	gtk_xtext_set_background (xtext, channelwin_pix, prefs.transparent, prefs.tint);
 	gtk_xtext_set_wordwrap (xtext, prefs.wordwrap);
+	gtk_xtext_set_show_marker (xtext, prefs.show_marker);
 	gtk_xtext_set_show_separator (xtext, prefs.indent_nicks ? prefs.show_separator : 0);
 	gtk_xtext_set_indent (xtext, prefs.indent_nicks);
 	if (!gtk_xtext_set_font (xtext, prefs.font_normal))
@@ -1914,7 +2029,7 @@ mg_create_userlist (session_gui *gui, GtkWidget *box, int pack)
 	if (prefs.style_namelistgad)
 	{
 		gtk_widget_set_style (ulist, input_style);
-		gtk_widget_modify_base (ulist, GTK_STATE_NORMAL, &colors[19]);
+		gtk_widget_modify_base (ulist, GTK_STATE_NORMAL, &colors[COL_BG]);
 	}
 
 	mg_create_meters (gui, vbox);
@@ -2106,6 +2221,30 @@ mg_switch_tab_cb (GtkWidget *tab, session *sess, gpointer family)
 		mg_show_generic_tab (active_tab);
 }
 
+/* compare two tabs (for tab sorting function) */
+
+static int
+mg_tabs_compare (session *a, session *b)
+{
+	/* this is for "Open Utilities in: Tabs" (i.e. sess is NULL) */
+	if (!a)
+		return 1;
+	if (!b)
+		return -1;
+
+	/* server tabs always go first */
+	if (a->type == SESS_SERVER)
+		return -1;
+
+	/* then channels */
+	if (a->type == SESS_CHANNEL && b->type != SESS_CHANNEL)
+		return -1;
+	if (a->type != SESS_CHANNEL && b->type == SESS_CHANNEL)
+		return 1;
+
+	return strcasecmp (a->channel, b->channel);
+}
+
 static void
 mg_create_tabs (session_gui *gui, GtkWidget *box)
 {
@@ -2114,7 +2253,8 @@ mg_create_tabs (session_gui *gui, GtkWidget *box)
 	if (prefs.tabs_position == 2 || prefs.tabs_position == 3)
 		vert = TRUE;
 
-	gui->tabs_box = tab_group_new (mg_switch_tab_cb, vert, FALSE);
+	gui->tabs_box = tab_group_new (mg_switch_tab_cb, mg_tabs_compare,
+											 vert, prefs.tab_sort);
 	gtk_table_attach (GTK_TABLE (gui->main_table), gui->tabs_box,
 						1, 2, 0, 1, GTK_FILL, GTK_FILL, 0, 0);
 }
@@ -2123,7 +2263,16 @@ static gboolean
 mg_tabwin_focus_cb (GtkWindow * win, GdkEventFocus *event, gpointer userdata)
 {
 	current_sess = current_tab;
-	plugin_emit_dummy_print (current_sess, "Focus Window");
+	if (current_sess)
+	{
+		gtk_xtext_check_marker_visibility (GTK_XTEXT (current_sess->gui->xtext));
+		plugin_emit_dummy_print (current_sess, "Focus Window");
+	}
+#ifndef WIN32
+#ifdef USE_XLIB
+	unflash_window (GTK_WIDGET (win));
+#endif
+#endif
 	return FALSE;
 }
 
@@ -2133,6 +2282,12 @@ mg_topwin_focus_cb (GtkWindow * win, GdkEventFocus *event, session *sess)
 	current_sess = sess;
 	if (!sess->server->server_session)
 		sess->server->server_session = sess;
+	gtk_xtext_check_marker_visibility(GTK_XTEXT (current_sess->gui->xtext));
+#ifndef WIN32
+#ifdef USE_XLIB
+	unflash_window (GTK_WIDGET (win));
+#endif
+#endif
 	plugin_emit_dummy_print (sess, "Focus Window");
 	return FALSE;
 }
@@ -2303,8 +2458,8 @@ mg_create_tabwindow (session *sess)
 	sess->gui->window = win;
 	gtk_window_move (GTK_WINDOW (win), prefs.mainwindow_left,
 						  prefs.mainwindow_top);
-/*	gtk_window_set_default_size (GTK_WINDOW (win), prefs.mainwindow_width,
-										  prefs.mainwindow_height);*/
+	if (prefs.gui_win_state)
+		gtk_window_maximize (GTK_WINDOW (win));
 	gtk_container_set_border_width (GTK_CONTAINER (win), 2);
 
 	g_signal_connect (G_OBJECT (win), "delete_event",
@@ -2315,6 +2470,8 @@ mg_create_tabwindow (session *sess)
 							G_CALLBACK (mg_tabwin_focus_cb), NULL);
 	g_signal_connect (G_OBJECT (win), "configure_event",
 							G_CALLBACK (mg_configure_cb), NULL);
+	g_signal_connect (G_OBJECT (win), "window_state_event",
+							G_CALLBACK (mg_windowstate_cb), NULL);
 
 	palette_alloc (win);
 
@@ -2392,6 +2549,7 @@ mg_add_generic_tab (char *name, char *title, void *family, GtkWidget *box)
 	but = tab_group_add (mg_gui->tabs_box, name, family, NULL,
 								mg_tab_press_cb, mg_link_cb, prefs.truncchans,
 								prefs.tab_dnd);
+	tab_set_attrlist (but, plain_list);
 	g_object_set_data (G_OBJECT (but), "title", strdup (title));
 	g_object_set_data (G_OBJECT (but), "box", box);
 	g_object_set_data (G_OBJECT (but), "sess", NULL);
@@ -2559,10 +2717,11 @@ fe_set_channel (session *sess)
 }
 
 void
-mg_changui_new (session *sess, restore_gui *res, int tab)
+mg_changui_new (session *sess, restore_gui *res, int tab, int focus)
 {
 	int first_run = FALSE;
 	session_gui *gui;
+	struct User *user = NULL;
 
 	if (!res)
 	{
@@ -2575,6 +2734,9 @@ mg_changui_new (session *sess, restore_gui *res, int tab)
 	if (!sess->server->front_session)
 		sess->server->front_session = sess;
 
+	if (!is_channel (sess->server, sess->channel))
+		user = find_name_global (sess->server, sess->channel);
+
 	if (!tab)
 	{
 		gui = malloc (sizeof (session_gui));
@@ -2583,6 +2745,8 @@ mg_changui_new (session *sess, restore_gui *res, int tab)
 		sess->gui = gui;
 		mg_create_topwindow (sess);
 		fe_set_title (sess);
+		if (user && user->hostname)
+			set_topic (sess, user->hostname);
 		return;
 	}
 
@@ -2602,9 +2766,13 @@ mg_changui_new (session *sess, restore_gui *res, int tab)
 		gui->is_tab = TRUE;
 	}
 
+	if (user && user->hostname)
+		set_topic (sess, user->hostname);
+
 	mg_add_chan (sess);
 
-	if (first_run || prefs.newtabstofront)
+	if (first_run || (prefs.newtabstofront == FOCUS_NEW_ONLY_ASKED && focus)
+			|| prefs.newtabstofront == FOCUS_NEW_ALL )
 		tab_focus (res->tab);
 
 /*	while (g_main_pending ())
